# This script extracts a series of measurements intended at an evaluation of 
# the quality of voice of laryngectomized speakers.
# The input sound is supposed to have the follwing structure:
# only one speaker who utters the following, IDENTIFIED in a companion TextGrid:
# - sustained /a/ vowel                => ID  "vogal"
# - one word with unvoiced plosives    => ID  "desvozeado"
# - one word with voices plosives      => ID  "vozeado"
# (the two words shall have the same syllabic structure, and only plosives as consonants: VCVCV)
# only consonnants between vowels will be considered
# - free speech                        => ID  "fala"
# - singing                            => ID  "canto"
# 
# NOTE: in this version,  the different parts of the speech recordings are interactively  annotated by the user
# so to help the analysis process
# 
# 
# The script estimates the following measurements:
# - the maximum duration of voicing (from the sustained vowel)
# - an indices of the quality of devoicing, comparing the amount of energy drop within the unvoiced/voiced consonants
# - the smooth Cepstrum Peak Prominence
# - the variation of F0 and intensity in speech and singin (mixed, until we find a way to separated both tasks)
# 
# 
# The script is under active development and its output were still NOT tested; sue it with care.
#
# The script output is currently done within Praat's Information windows
# TODO: latter versions shall output a result file
# 
# 
# 
# 
# Script written by Albert Rilliard (Université Paris Saclay, CNRS, LISN) in collaboration with Zuleica Camargo & Nathalia dos Reis (PUC-SP)
# Contact: albert.rilliard@limsi.fr, zcamargo@pucsp.br
# 
# This script is free software distributed under the CeCILL FREE SOFTWARE LICENSE AGREEMENT version 2.1 or higher (compatible with GNU GPL)
# See https://cecill.info/licences/Licence_CeCILL_V2.1-en.html for details
# 
# Research funded by:
# pesquisa PIPEq- Auxílio à Pesquisa- AuxP- 
# Projeto ENTRE A PRODUÇÃO E A PERCEPÇÃO DA FALA TRAQUEOESOFÁGICA: DEMANDAS CLÍNICAS) nº ( Nº 21945)
# 
# version 0.3 - 14/06/2022
# 
############################################


# in future versions, the script shall (?) proceed to the recording of the targetted sounds, 
# so to avoid having to infer a segmentation from silences estimated from a random file...
# => we choose to provide manual segmentation for now
# this version  guide the segmentation process, asking for the identification of each part


###############
# check PRAAT version: to use CPPS, it seems better to use a version > 6.1.37 (december 2020)
if "6.1.37" < praatVersion$
  writeInfoLine: "Processing done with Praat version " + praatVersion$
else
  exitScript: "This Praat version seems too old for running this script. You are using Praat " + praatVersion$ + ". It would be better to use a version published after Praat 6.1.37 (visit praat.org)."
endif

###############
# information on the manual file selection
beginPause: "Input file selection"
  comment: "You'll be asked to select one sound file, that contains the voice to be analyzed; "
  comment: "The sound shall have five parts:"
  comment: "A sustained vowel, two words with un/voiced CVCV structure,"
  comment: "free speech, singing."
  comment: "You'll then be asked to identiiy these different parts."
  comment: "Please click Start to continue"
endPause: "Start", 1

# TODO: latter a menu will help all this
# Open the target sound (ask the user to select a sound file)
filename$ = chooseReadFile$: "Select the sound to analyse"

assert filename$ <> ""
wavin = Read from file: filename$


# NO MORE : it is now done in an interactive series of questions that construct the TextGrid for latter process
# we ask for the TextGrid -- may be open autommatically if in the same folder same name, but we don't know
#tgname$ = chooseReadFile$: "Select the associated TextGrid file"
#assert tgname$ <> ""
#tgin = Read from file: tgname$


# Create a TextGrid to be field with the required information
selectObject: wavin
tgin = To TextGrid: "intervals points", "points"




###############
# Interaction to retrieve the five parts of the recording


#
# sustained vowel: select it
#
selectObject: wavin
View & Edit
beginPause: "Sustained vowel"
  comment: "Please select the sustained vowel, from its begining to end."
  comment: "The selected part will be used to infer the maximumu phonation time,"
  comment: "limiting it to 15dB below the max found intensity value."
  comment: " "
  comment: "When done, please click 'Ok'."
  comment: "If there is no sustained vowel, click 'No vowel'."
res_voy = endPause: "Ok", "No vowel", 1

if res_voy = 2
  isvoy = 0
  debv = 0
  endv = 0
else
  isvoy = 1
  editor: wavin
    debv = Get start of selection
    endv = Get end of selection
  endeditor
  selectObject: tgin
  Insert boundary: 1, debv
  Insert boundary: 1, endv
  medv = ( debv + endv ) / 2
  idv = Get interval at time: 1, medv
  Set interval text: 1, idv, "vogal"
endif

editor: wavin
  Close
endeditor


# TARGET WORDS
selectObject: wavin
View & Edit

# CVCV unvoiced
# middle of the tonic consonnant

beginPause: "Unvoiced word"
  comment: "UNVOICED word"
  comment: "Please click at the middle of the tonic consonant of the "
  comment: "unvoiced target word (e.g. the /p/ of /pata/)"
  comment: " "
  comment: "When done, please click 'Ok'."
  comment: "If there is no unvoiced target word, click 'No unvoiced target'."
res_uwc = endPause: "Ok", "No unvoiced target", 1
if res_uwc = 2
  isuwc = 0
  tpuwc = 0
  tpuwv = 0
else
  isuwc = 1
  editor: wavin
    tpuwc = Get cursor
  endeditor
  selectObject: tgin
  Insert point: 2, tpuwc, "unvoiced_C"
  
  # middle of the tonic vowel
  # 
  # select the middle of the Vowel for the unvoiced target word (the word exist, no further check)
  beginPause: "Unvoiced word"
    comment: "UNVOICED word"
    comment: "Now click at the middle of the tonic VOWEL of the "
    comment: "unvoiced target word (e.g. the first /a/ of /pata/)"
    comment: " "
    comment: "When done, please click 'Ok'."
  endPause: "Ok", 1
  editor: wavin
    tpuwv = Get cursor
  endeditor
  selectObject: tgin
  Insert point: 2, tpuwv, "unvoiced_V"
endif

# CVCV voiced
# middle of the tonic consonnant

beginPause: "Voiced word"
  comment: "VOICED word"
  comment: "Please click at the middle of the tonic consonant of the "
  comment: "VOICED target word (e.g. the /b/ of /bata/)"
  comment: " "
  comment: "When done, please click 'Ok'."
  comment: "If there is no voiced target word, click 'No voiced target'."
res_vwc = endPause: "Ok", "No voiced target", 1
if res_vwc = 2
  isvwc = 0
  tpvwc = 0
  tpvwv = 0
else
  isvwc = 1
  editor: wavin
    tpvwc = Get cursor
  endeditor
  selectObject: tgin
  Insert point: 2, tpvwc, "voiced_C"
  
  # middle of the tonic vowel (Voiced)
  # 
  # select the middle of the Vowel for the voiced target word (the word exist, no further check)
  beginPause: "Voiced word"
    comment: "VOICED word"
    comment: "Now click at the middle of the tonic VOWEL of the "
    comment: "VOICED target word (e.g. the first /a/ of /bata/)"
    comment: " "
    comment: "When done, please click 'Ok'."
  endPause: "Ok", 1
  editor: wavin
    tpvwv = Get cursor
  endeditor
  selectObject: tgin
  Insert point: 2, tpvwv, "voiced_V"
endif

# close the editor and open a new for the last two parts
editor: wavin
  Close
endeditor




# 
# free speech: select begining / end 
# 
selectObject: wavin
View & Edit
beginPause: "Free speech"
  comment: "Please select the part of recording that contains free speech (beware selecting only one speaker)"
  comment: "from its begining to end. The selected part will be used to infer dynamic capacity of the voice."
  comment: "It will also be used to estimate the smoothed CPP."
  comment: " "
  comment: "When done, please click 'Ok'."
  comment: "If there is no free speech, click 'No free speech'."
res_sp = endPause: "Ok", "No free speech", 1

if res_sp = 2
  issp = 0
  debs = 0
  ends = 0
else
  issp = 1
  editor: wavin
    debs = Get start of selection
    ends = Get end of selection
  endeditor
  selectObject: tgin
  Insert boundary: 1, debs
  Insert boundary: 1, ends
  meds = ( debs + ends ) / 2
  ids = Get interval at time: 1, meds
  Set interval text: 1, ids, "fala"
endif
#Close

# singing: select begining / end
# 
#selectObject: wavin
#View & Edit

beginPause: "Singing"
  comment: "Now select the part of recording that contains singing (beware selecting only one speaker)"
  comment: "from its begining to end. The selected part will be used to infer dynamic capacity of the voice."
  comment: " "
  comment: "When done, please click 'Ok'."
  comment: "If there is no singing part, click 'No song'."
res_sg = endPause: "Ok", "No song", 1

if res_sg = 2
  issg = 0
  debg = 0
  endg = 0
else
  issg = 1
  editor: wavin
    debg = Get start of selection
    endg = Get end of selection
  endeditor
  selectObject: tgin
  Insert boundary: 1, debg
  Insert boundary: 1, endg
  medg = ( debg + endg ) / 2
  idg = Get interval at time: 1, medg
  Set interval text: 1, idg, "canto"
endif
editor: wavin
  Close
endeditor


### The textGrid shall be complete: 
# we open it so the user can have a chance to check and make change if needed.


selectObject: wavin
plusObject: tgin
View & Edit

beginPause: "Check"
  comment: "Please check if the five parts are adequately annnotated in the TextGrid:"
  comment: " (sustained vowel, un/voiced consonants and vowels, free speech, song)"
  comment: "Make change if required."
  comment: " "
  comment: "When done, please click 'Proceed': the measurements will be estimated"
endPause: "Proceed", 1

editor: tgin
  Close
endeditor





###############
# Test if the different parts in the sound files that shall correspond to the four parts - exist (in some case, some parts are not present):
# 1- sustained vowel
# 2- two VCVCV words
# 3- free speech 
# 4- singing
# 


# Gives information on the measurements
appendInfoLine: "Analysis of sound " + filename$
appendInfoLine: "(will analyze a sustained vowel, two VCV words (voiced/unvoiced), and then free speech and singing, if available)"


# we check for each part if it is present - and process it or leave a message


###############
# SUSTAINED VOWEL
isvoy = 0
selectObject: tgin
nbsdint = Count intervals where: 1, "is equal to", "vogal"
if nbsdint < 1
  appendInfoLine: ""
  appendInfoLine: " * There is no interval labeled 'vogal' in the TextGrid: the sustained vowel will not be processed"
  appendInfoLine: ""
else
  if nbsdint > 1
  appendInfoLine: ""
    appendInfoLine: " * There are MORE than ONE interval labeled 'vogal' in the TextGrid: "
    appendInfoLine: "the sustained vowel will not be processed (correct the TextGrid and launch again)"
    appendInfoLine: ""
  else
    # OK process the vowel
    isvoy = 1
    # sustained vowel
    selectObject: wavin
    plusObject: tgin
    vow = Extract intervals where: 1, "no", "is equal to", "vogal"
    
    ## for the sustained vowel 
    selectObject: vow
    
    # TODO : check the segmentation as max intensity minus 15dB ?? (ver se faz sentido)
    # we shall either keep the segmentation given by the experimenter, or restrict the interval to a given rule:
    # consider the vowel duration as the duration of the parts that have at least 15dB from the max intensity
    # may be selected in the fiture with the menu?
    if 1 < 2
      intA = To Intensity: 30, 0.005, "yes"
      tgA = To TextGrid (silences): -15, 0.01, 0.07, "silent", "sounding"
      durv = Get total duration of intervals where: 1, "is equal to", "sounding"
      selectObject: intA
      plusObject: tgA
      Remove
    else
      selectObject: vow
      durv = Get total duration
    endif
    # print the result
    appendInfoLine: " "
    appendInfoLine: " * Measures for the sustained vowel:"
    appendInfoLine: "Sustained vowel duration: " + string$(round(1000*durv)/1000) + " seconds"
    
  endif
endif


###############
# VOICED / UNVOICED VCV WORDS
isvuv = 0
selectObject: tgin
#nbsdint = Count intervals where: 1, "is equal to", "desvozeado"
nbunC = Count points where: 2, "is equal to", "unvoiced_C"
nbunV = Count points where: 2, "is equal to", "unvoiced_V"
nbvoC = Count points where: 2, "is equal to", "voiced_C"
nbvoV = Count points where: 2, "is equal to", "voiced_V"

# check if there are missing labels
if nbunC < 1
  appendInfoLine: ""
  appendInfoLine: "* There is no unvoiced Consonnant in the TextGrid: voiced/unvoiced contrast will not be processed"
  appendInfoLine: ""
else
if nbunV > 1
  appendInfoLine: ""
  appendInfoLine: "* There is no vowel for the unvoiced word labelled in the TextGrid: voiced/unvoiced contrast will not be processed"
  appendInfoLine: ""
else
if nbvoC > 1
  appendInfoLine: ""
  appendInfoLine: "* There is no voiced Consonnant in the TextGrid: voiced/unvoiced contrast will not be processed"
  appendInfoLine: ""
else
if nbvoV > 1
  appendInfoLine: ""
  appendInfoLine: "* There is no vowel for the voiced word labelled in the TextGrid: voiced/unvoiced contrast will not be processed"
  appendInfoLine: ""
# ok
      else
        # OK process the VCVs
        isvuv = 1
        # VCV word with unvoiced plosives
        # time of the four measuurement points
        # tpuwc  tpuwv  tpvwc  tpvwv 
        # From the VCV unvoiced sounds, extract an intensity object with large averaging windows
        selectObject: wavin
        int_in = To Intensity: 30, 0.005, "yes"
        
        # from this object estimate the value at each of the four points
        # measure on windows of 30ms around the measure points (median)
        selectObject: int_in
        
        tin = tpuwc - 0.015
        tou = tpuwc + 0.015
        int_unC = Get quantile: tin, tou, 0.5
        
        tin = tpuwv - 0.015
        tou = tpuwv + 0.015
        int_unV = Get quantile: tin, tou, 0.5
        
        tin = tpvwc - 0.015
        tou = tpvwc + 0.015
        int_voC = Get quantile: tin, tou, 0.5
        
        tin = tpvwv - 0.015
        tou = tpvwv + 0.015
        int_voV = Get quantile: tin, tou, 0.5
        
        #diff_int_unv = mean_peak_un - mean_val_un
        #diff_int_voi = mean_peak_vo - mean_val_vo
        diff_int_unv = int_unV - int_unC
        diff_int_voi = int_voV - int_voC
        
        # print the estimation of intensity difference within VCV words
        appendInfoLine: " "
        appendInfoLine: " * Measures for the voiced/unvoiced difference:"
        appendInfoLine: "V/C unvoiced intensity difference: " + string$(round(100*diff_int_unv)/100) + "dB"
        appendInfoLine: "V/C voiced intensity difference:   " + string$(round(100*diff_int_voi)/100) + "dB"
        
      endif
    endif
  endif
endif


###############
# FREE SPEECH
issp = 0
selectObject: tgin
nbsdint = Count intervals where: 1, "is equal to", "fala"
if nbsdint < 1
  appendInfoLine: ""
  appendInfoLine: " * There is no interval labeled 'fala' in the TextGrid: the free speech part will not be processed"
  appendInfoLine: ""
else
  if nbsdint > 1
    appendInfoLine: ""
    appendInfoLine: " * There are MORE than ONE interval labeled 'fala' in the TextGrid: correct the TextGrid and launch again"
    appendInfoLine: ""
  else
    # OK process the speech
    issp = 1
    # extract all the other "souding" parts to group them in a large sound to extract the other parameters 
    # (CPP; F0 and intensity variation)
    # extraction of all other sounding sounds to concatenate them
    selectObject: wavin
    plusObject: tgin
    
    fala = Extract intervals where: 1, "no", "is equal to", "fala"
    # let's remove the silent parts fo speech 
    tgsp = To TextGrid (silences): 40, 0.005, -25, 0.25, 0.05, "silent", "sounding"
    plusObject: fala
    falas# = Extract intervals where: 1, "no", "is equal to", "sounding"
    falaconcat = Concatenate
    selectObject (falas#)
    Remove

    ## for the spontaneous speeech
    # estimation of intensity variation
    
    selectObject: falaconcat
    #int_fala = To Intensity: 100, 0, "yes"
    int_fala = To Intensity: 30, 0, "yes"
    
    selectObject: int_fala
    int_fala_p = To IntensityTier (peaks)
    int_fala_tr = Down to TableOfReal
    
    fala_int_peak_mean = Get column mean (index): 2
    fala_int_peak_std = Get column stdev (index): 2
    
    appendInfoLine: " "
    appendInfoLine: " * Measures for the spoken part:"
    appendInfoLine: "Speech intensity mean: " + string$(round(100*fala_int_peak_mean)/100) + "  (non calibrated)"
    appendInfoLine: "Speech intensity std  deviation: " + string$(round(100*fala_int_peak_std)/100) + "dB"
    
    
    # estimation of  F0 variation -- if there is some available F0 ???
    # it does not seems to be reliable!!!!
    
  endif
endif


###############
# SINGING
issg = 0
selectObject: tgin
nbsdint = Count intervals where: 1, "is equal to", "canto"
if nbsdint < 1
  appendInfoLine: ""
  appendInfoLine: " * There is no interval labeled 'canto' in the TextGrid: the singing part will not be processed"
  appendInfoLine: ""
else
  if nbsdint > 1
    appendInfoLine: ""
    appendInfoLine: " * There are MORE than ONE interval labeled 'canto' in the TextGrid: correct the TextGrid and launch again"
    appendInfoLine: ""
  else
    # OK process the song
    issg = 0
    # extract all the other "souding" parts to group them in a large sound to extract the other parameters 
    # (CPP; F0 and intensity variation)
    # extraction of all other sounding sounds to concatenate them
    selectObject: wavin
    plusObject: tgin
    
    canto = Extract intervals where: 1, "no", "is equal to", "canto"
    # let's remove the silent parts fo speech 
    tgsg = To TextGrid (silences): 40, 0.005, -25, 0.25, 0.05, "silent", "sounding"
    plusObject: canto
    cantos# = Extract intervals where: 1, "no", "is equal to", "sounding"
    cantoconcat = Concatenate
    selectObject (cantos#)
    Remove

    ## for the spontaneous speeech
    # estimation of intensity variation
    
    selectObject: cantoconcat
    #int_sing = To Intensity: 100, 0, "yes"
    int_sing = To Intensity: 30, 0, "yes"
    
    selectObject: int_sing
    int_sing_p = To IntensityTier (peaks)
    int_sing_tr = Down to TableOfReal
    
    sing_int_peak_mean = Get column mean (index): 2
    sing_int_peak_std = Get column stdev (index): 2
    
    appendInfoLine: " "
    appendInfoLine: " * Measures for the singing part:"
    appendInfoLine: "Singing intensity mean: " + string$(round(100*sing_int_peak_mean)/100) + "  (non calibrated)"
    appendInfoLine: "Singing intensity std  deviation: " + string$(round(100*sing_int_peak_std)/100) + "dB"
    
    # estimation of  F0 variation -- if there is some available F0 ???
    # it does not seems to be reliable!!!!
    
  endif
endif



###############
# CPP
# write a note that the process is long but it is normal

# Check there is Speech PLUS vowel
vpluss = isvoy + issp

if vpluss < 2
  appendInfoLine: ""
  appendInfoLine: ""
  appendInfoLine: "Need BOTH free speech and a sustained vowel to estimate the CPPS (it is not the case here)."
else
  appendInfoLine: ""
  appendInfoLine: ""
  appendInfoLine: " -- start the estimation of CPPS; it takes some time (minutes), please be patient --"
  
  selectObject: falaconcat
  plusObject: vow
  # if one wants to include the singing part
  # plusObject: cantoconcat
  vpf = Concatenate
  
  
  # transform the speech waveform into cepstrogram (default parameters, but for lower F0 min)
  selectObject: vpf
  pcsp = To PowerCepstrogram: 40, 0.002, 5000, 50
  # calculate the smoothed CPP (see the publication of weenink for parameters)
  cpps = Get CPPS: "yes", 0.02, 0.0005, 40, 200, 0.05, "parabolic", 0.001, 0.05, "Exponential decay", "Robust slow"
  
  # if one want CPP on single frames
  #selectObject: pcsp
  #To PowerCepstrum (slice): 0.1
  #Get peak prominence: 60, 333.3, "parabolic", 0.001, 0.05, "Exponential decay", "Robust slow"
  
  # display the result
  appendInfoLine: " "
  appendInfoLine: "CPPS (on sustained vowel plus speech parts): " + string$(round(100*cpps)/100) + "dB"
endif




